-- save pending data, stop scanning and reset runtime variables
function PR.FreshStart()
    PR.ScannerOffSwitch()
    PR.SaveNow()
    PR.LastDisplayedRoidId = 0
    PR.MassTarget = 0
    PR.MassTestActive = false
    PR.MassScanCount = 0
    PR.MassScanMax = 0
    PR.LastTemp = 0
    PR.RoidNode = 0
end

-- event handler for SECTOR_CHANGED
function PR.SectorChanged(_, sectorId)
    -- Known issue: GetCurrentSectorid() returns nil after a ReloadInterface()
    sectorId = tonumber(sectorId) or GetCurrentSectorid()
    if not sectorId then
        PR.Print(PR.PluginTitle ..
            " detected a bug with the game, please restart (you are likely about to crash)",
            true)
        PR.PluginReady = false
        return
    end
    -- first cleanup any left-overs
    PR.FreshStart()
    -- then prepare new sector
    PR.Sector = sectorId
    PR.LoadSector(PR.Sector)
    PR.StatusPrint()
end

-- save sector and corresponding system summary data
-- set optional differentSectorOnly to true to prevent saving current sector
function PR.SaveNow(differentSectorOnly)
    -- force a pending sector data update
    if PR.SaveSectorId > 0 and
        (not differentSectorOnly or PR.SaveSectorId ~= PR.Sector) then
        PR.SaveSectorNow()
    end
    -- force a pending update of system ore summary data
    if PR.SystemUpdateSectorId > 0 and
        (not differentSectorOnly or PR.SystemUpdateSectorId ~= PR.Sector) then
        PR.SystemDataUpdateNow()
    end
end

-- save PR.Config
function PR.SaveConfigNow()
    SaveSystemNotes(spickle(PR.Config), PR.SystemNotesPrefix)
end

-- redirects output to HUD on PC, if HUD is showing, and forceChat is not true
function PR.Print(text, forceChat)
    text = PR.Color.Magenta .. tostring(text) .. PR.Color.TERMINATOR
    if not PR.Config.COLOR then
        text = PR.Color.White .. filter_colorcodes(text) .. PR.Color.TERMINATOR
    end
    if not forceChat and PR.Config.PrintToHUD and HUD.visible then
        HUD:PrintSecondaryMsg("\t" .. text)
    else
        print(text)
    end
end

-- returns short ore name for provided full name based on PR.Ore,
-- otherwise full name
function PR.GetOreNameShort(longore)
    for i = 1, PR.Ore.n do
        if PR.Ore[i].name == longore then return PR.Ore[i].shortname end
    end
    return longore
end

-- returns full ore name for provided short name based on PR.Ore,
-- otherwise short name
function PR.GetOreName(shortore)
    for i = 1, PR.Ore.n do
        if PR.Ore[i].shortname == shortore then return PR.Ore[i].name end
    end
    return shortore
end

-- return number of digits of provided integer number
function PR.GetDigitCount(number)
    local digits = 1
    local power_of_10 = 10
    while (power_of_10 <= number) do
        digits = digits + 1
        power_of_10 = power_of_10 * 10
    end
    return digits
end

-- returns a colored string indicating what types of bots are sighted -
-- doubles as a station detector via Station Guard bots (except training
-- systems, but we don't care about those since you cannot scan or mine there)
function PR.GetBotsSighted(sectorId)
    local bots
    -- use my Botwatcher plugin if it's installed
    if PR.IsBotwatcherInstalled then
        bots = Botwatcher.GetBotsSightedList(sectorId)
    else
        local bots_sighted = GetBotSightedInfoForSector(sectorId)
        if bots_sighted ~= "\n" then
            bots = {}
            for bot in bots_sighted:sub(14):gmatch(" (.-)[,\n]") do
                table.insert(bots, bot)
            end
        end
    end
    if bots then
        for _, bot in ipairs(bots) do
            if bot == "Station Guard" then
                return PR.Color.Green .. "  Station" .. PR.Color.TERMINATOR
            elseif bot ~= "Milanar Guardian" then -- Milanar Guardian is really a collector
                local suffix = bot:sub(-9)
                if suffix ~= "Collector" and suffix ~= "Transport" and
                    suffix ~= " Observer" then
                    return PR.Color.Red .. "  Combat bots" ..
                        PR.Color.TERMINATOR
                end
            end
        end
        return PR.Color.Gold .. "  Collector bots" .. PR.Color.TERMINATOR
    end
    return ""
end
